@@ -1,8 +1,8 @@
-## plot.loci.R (2009-05-10)
+## plot.loci.R (2018-08-29)
 
 ##   Plot Loci Frequencies
 
-## Copyright 2009 Emmanuel Paradis
+## Copyright 2009-2018 Emmanuel Paradis
 
 ## This file is part of the R-package `pegas'.
 ## See the file ../DESCRIPTION for licensing issues.
@@ -12,9 +12,9 @@ plot.summary.loci <-
 {
     what <- match.arg(what, c("both", "alleles", "genotypes"))
     layout(matrix(1:layout, ceiling(sqrt(layout))))
-    if (!par("ask")) {
-        par(ask = TRUE)
-        on.exit(par(ask = FALSE))
+    if (!devAskNewPage() && !names(dev.cur()) %in% c("pdf", "postscript")) {
+        devAskNewPage(TRUE)
+        on.exit(devAskNewPage(FALSE))
     }
     nms <- names(x)
     N <- if (!missing(loci)) match(loci, nms) else seq_along(x)
